#include <iostream>
#include <fstream>
#include <vector>
#include <queue>
#include <cmath>
#include "alpini.h"
using namespace std;

/*
 * Soluzione con rimozione probalistica:
 *  - rimuove il nodo di grado maggiore
 *  - rimuove nodi estratti a caso con probabilità dipendente dal grado
 *
 * (33.78 punti)
 */

ifstream in("input.txt");
ofstream out("output.txt");

int N, M;
int best;

struct nodo{
  vector<int> vic;
  bool destroyed = false;
  bool visited = false;
  int degree = 0;
};

vector<nodo> g;

/*
 * Visita una componente connessa del grafo, se trova un nodo già visitato
 * vuol dire che è stato individuato un ciclo e restituisce true, altrimenti
 * restituisce false.
 */
bool trova_ciclo(int n, int f, vector<bool>& vis){
  vis[n] = true;
  for(int v: g[n].vic){
    if(v != f && !g[v].destroyed){
      if(vis[v])
        return true;
      if(trova_ciclo(v, n, vis))
        return true;
    }
  }
  return false;
}

/*
 * Visita tutte le componenti connesse del grafo, se non ci sono cicli
 * restituisce true, altrimenti restituisce false
*/
bool no_cicli(){
  vector<bool> vis(N, false);
  for(int i=0;i<N;i++)
    if(!vis[i] && !g[i].destroyed)
      if(trova_ciclo(i, i, vis))
        return false;
  return true;
}

/*
 * Rimuove le foglie, ovvero i nodi che non appartengono ad alcun ciclo
 */
void clean_leaves(){
  queue<int> q;
  for(int i=0;i<N;i++)
    if(!g[i].destroyed && g[i].degree == 1)
      q.push(i);
  while(!q.empty()){
    int v = q.front();
    q.pop();
    g[v].degree = 0;
    for(int vv:g[v].vic){
      if(!g[vv].destroyed){
        g[vv].degree--;
        if(g[vv].degree == 1)
          q.push(vv);
      }
    }
  }
}

void rec_greedy(bool forced, int pot){

  // ogni volta che la funzione viene chiamata provo a rimuovere un insieme
  // diverso di nodi.
  for(auto& n:g){
    n.destroyed = false;
    n.degree = n.vic.size();
  }

  // continuo a rimuovere nodi finché ci sono 
  int n = 0;
  while(!no_cicli() && n < best-1){
    // rimuove le foglie
    clean_leaves();
    int next = 0;

    if(forced){
      // Prima chiamata (riga 164)
      // seleziona il nodo di grado maggiore che non è già stato marcato come
      // da distruggere
      int b = -1;
      for(int i=0;i<N;i++)
        if(!g[i].destroyed && (b==-1 || g[i].degree > g[b].degree))
          b = i;
      next = b;
    }else{
      // Chiamate successive (riga 166)

      // calcola D = \sum_{node} (degree(node))^pot
      int totd = 0;
      for(auto& no:g)
        if(!no.destroyed){
          totd+= pow(no.degree, pot);
        }

      // estraggo un numero casuale [0, D): c
      int choice = rand()%totd;

      // quando incontro un nodo per cui grado cumulativo D' di quelli che
      // non ho ancora distrutto è > c, marco il nodo come da distruggere 
      int cur = 0;
      next = -1;
      for(int i=0;i<N;i++){
        if(!g[i].destroyed){
          cur += pow(g[i].degree, pot);
          if (cur > choice){
            next = i;
            break;
          }
        }
      }
    }
    // distruggo il nodo considerato precedentemente e aggiorno il grado
    g[next].destroyed = true;
    n++;
    for(int v: g[next].vic)
      g[v].degree--;
  }

  // non ci sono più cicli, stampo il risultato
  if(no_cicli()){
    out<<n;
    for(int i=0;i<N;i++)
      if(g[i].destroyed){
        g[i].destroyed = false;
        out<<" "<<i;
      }
    out<<"#"<<endl;
    best = n;
  }
}

int main() {
  helpers::setup();
  int D = 0;

  in >> N >> M;
  g.resize(N);
  best = N+1;
  for(int i=0;i<M;i++){
    int f, t;
    in >> f >> t;
    g[f].vic.push_back(t);
    g[f].degree ++;
    g[t].vic.push_back(f);
    g[t].degree ++;
  }
  rec_greedy(true, 0);
  while(true)
    rec_greedy(false,5);
  return 0;
}
